package com.huimv.manage.eartag.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.huimv.manage.eartag.dao.entity.EtEarmarkEntity;
import com.huimv.manage.eartag.dao.entity.EtPackageEntity;
import com.huimv.manage.eartag.dao.repo.EarmarkRepo;
import com.huimv.manage.eartag.dao.repo.PackageRepo;
import com.huimv.manage.eartag.service.IEarmarkService;
import com.huimv.manage.util.*;
import com.huimv.manage.webservice.Soap;
import com.huimv.manage.webservice.task.EarmarkTask;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.util.ResourceUtils;
import org.springframework.util.StringUtils;

import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.sql.Timestamp;
import java.util.*;

import org.apache.commons.compress.archivers.ArchiveException;
import org.apache.commons.compress.archivers.ArchiveOutputStream;
import org.apache.commons.compress.archivers.ArchiveStreamFactory;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
//import org.apache.tomcat.util.http.fileupload.FileUtils;
import org.apache.commons.io.FileUtils;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class EarmarkServiceImpl implements IEarmarkService {
    @Value("${webservice.url}")
    private String webServiceUrl;
    @Autowired
    private EarmarkRepo earmarkRepo;
    @Autowired
    private EarmarkTask earmarkTask;
    @Autowired
    private Soap soap;
    @Autowired
    private CodeUtil codeUtil;
    @Autowired
    private ExcelUtil excelUtil;
    @Autowired
    private DataUtil dateUtil;
    @Autowired
    private TxtUtil txtUtil;

    @Autowired
    private PackageRepo packageRepo;


    /**
     * @Method : listEarmark
     * @Description : 展示列表
     * @Params : [pageSize]
     * @Return : com.huimv.manage.util.Result
     * @Author : ZhuoNing
     * @Date : 2021/10/29
     * @Time : 21:14
     */
    @Override
    public Result listEarmark(int pageSize) {
        //
        List<EtEarmarkEntity> earmarkEntityList = earmarkRepo.listEarmark(pageSize);
        return new Result(ResultCode.SUCCESS, earmarkEntityList);
    }

    @Override
    public Result listEarmark(Integer applyId, Integer packageId, Integer earMarkId, Integer earMarkNumber, Integer returnState, Integer writeState, Integer printState, Date startDate, Date endDate, Integer pageSize, Integer pageNo) {
        Specification<EtEarmarkEntity> sf = (Specification<EtEarmarkEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != applyId) {
                predList.add(criteriaBuilder.equal(root.get("applyId").as(Integer.class), applyId));
            }
            if (null != packageId) {
                predList.add(criteriaBuilder.equal(root.get("packageId").as(Integer.class), packageId));
            }
            if (null != earMarkId) {
                predList.add(criteriaBuilder.equal(root.get("earMarkId").as(Integer.class), earMarkId));
            }
            if (null != earMarkNumber) {
                predList.add(criteriaBuilder.equal(root.get("earMarkNumber").as(Integer.class), earMarkNumber));
            }
            if (null != startDate) {
                predList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("startDate").as(Date.class), startDate));
            }
            if (null != endDate) {
                predList.add(criteriaBuilder.lessThanOrEqualTo(root.get("endDate").as(Date.class), endDate));
            }
            if (null != returnState) {
                predList.add(criteriaBuilder.equal(root.get("returnState").as(Integer.class), returnState));
            }
            if (null != writeState) {
                predList.add(criteriaBuilder.equal(root.get("writeState").as(Integer.class), writeState));
            }
            if (null != printState) {
                predList.add(criteriaBuilder.equal(root.get("printState").as(Integer.class), printState));
            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        //分页
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, earmarkRepo.findAll(sf, pageable));
    }

    /**
     * @Method : getEarmark
     * @Description : 下载耳标号
     * @Params : [applyId]
     * @Return : com.huimv.manage.util.Result
     * @Author : ZhuoNing
     * @Date : 2021/10/29
     * @Time : 21:14
     */
    @Override
    public Result getEarmark(int applyId) throws IOException {
        // 构造请求
        String requestXML = earmarkTask.getRequestEarmarkXML(applyId);
//        System.out.println("requestXML="+requestXML);

        // 调用接口返回结果
        String earMarkReponseXML = soap.callSoap(requestXML, webServiceUrl);
//        System.out.println("earMarkReponseXML="+earMarkReponseXML);

        // 解析结果
        JSONArray downloadEarmarkJa = earmarkTask.parseEarMarkReponseXML(earMarkReponseXML);
        // 耳标入库
        JSONArray newEarmarkJa = filterEarmark(downloadEarmarkJa);
        if (newEarmarkJa.size() == 0) {
            return new Result(Const.CODE_NO_NEW_EARMARK, Const.NO_NEW_EARMARK, false);
        }
        for (int a = 0; a < newEarmarkJa.size(); a++) {
            JSONObject earmarkJo = newEarmarkJa.getJSONObject(a);
            EtEarmarkEntity earmarkEntity = new EtEarmarkEntity();
            earmarkEntity.setEarmarkId(earmarkJo.getString("earmarkId"));
            earmarkEntity.setPackageId(earmarkJo.getString("packageId"));
            earmarkEntity.setEarmarkNumber(earmarkJo.getString("earmarkNumber"));
            earmarkEntity.setApplyId(earmarkJo.getString("applyId"));
            earmarkEntity.setEarmarkData(codeUtil.convertBase64BinaryToHex(earmarkJo.getString("earmarkData")));
            earmarkEntity.setSetPrintState(0);
            earmarkEntity.setSetWriteState(0);
            earmarkEntity.setSetDownState(0);
            earmarkEntity.setSetUploadState(0);
            earmarkRepo.saveAndFlush(earmarkEntity);
        }
        return new Result(Const.CODE_OK, "本次下载" + newEarmarkJa.size() + "个新耳标号数据.", false);
    }

    /**
     * @Method : setUpdateDownState
     * @Description : 设置耳标已下载状态
     * @Params : [earmarkIDs]
     * @Return : com.huimv.manage.util.Result
     * @Author : ZhuoNing
     * @Date : 2021/10/29
     * @Time : 23:00
     */
    @Override
    public Result setUpdateDownState(String earmarkIDs) throws UnsupportedEncodingException {
        //构造请求字符串
        String requestXML = earmarkTask.getUpdateDownStateRequestXML(earmarkIDs);
        //远程调用返回结果
        String reponseXML = "";
        reponseXML = soap.callSoap(requestXML, webServiceUrl);
        // test
//        reponseXML = earmarkTask.TestParseUpdateDownStateXML();
        //解析
        String updateDownState = earmarkTask.parseUpdateDownStateXML(reponseXML);
        // 入库
        if (updateDownState.trim().equalsIgnoreCase("false")) {
            return new Result(Const.CODE_UPDATE_EARMARK_DOWN_STATE_FAIL, Const.UPDATE_EARMARK_DOWN_STATE_FAIL, false);
        } else {
            String[] earmarkIDArray = earmarkIDs.split(",");
            for (int a = 0; a < earmarkIDArray.length; a++) {
                EtEarmarkEntity earmarkEntity = new EtEarmarkEntity();
                earmarkEntity.setEarmarkId(earmarkIDArray[a]);
                ExampleMatcher matcher = ExampleMatcher.matching()
                        .withMatcher("earmark_id", ExampleMatcher.GenericPropertyMatchers.contains());
                Example example = Example.of(earmarkEntity, matcher);
                Optional optional = earmarkRepo.findOne(example);
                if (optional.isPresent()) {
                    EtEarmarkEntity earmarkEntity1 = (EtEarmarkEntity) optional.get();
                    earmarkEntity1.setSetDownState(1);
//                    earmarkEntity1.setSetDownDate(new Timestamp(new Date().getTime()));
                    earmarkEntity1.setSetDownDate(new Timestamp(System.currentTimeMillis()));
                    earmarkRepo.saveAndFlush(earmarkEntity1);
                } else {
                    return new Result(Const.CODE_NO_RECORD, Const.NO_RECORD, false);
                }
            }
            return new Result(ResultCode.SUCCESS);
        }
    }

    /**
     * @Method : setProduceState
     * @Description : 设置生产状态
     * @Params : [ids]
     * @Return : com.huimv.manage.util.Result
     * @Author : ZhuoNing
     * @Date : 2021/10/29
     * @Time : 23:56
     */
    @Override
    public Result setProduceState(String ids, int state) {
        String[] idArray = ids.split(",");
        for (int a = 0; a < idArray.length; a++) {
            Optional optional = earmarkRepo.findById(Integer.parseInt(idArray[a]));
            EtEarmarkEntity earmarkEntity = (EtEarmarkEntity) optional.get();
            earmarkEntity.setSetWriteState(state);
            earmarkEntity.setSetWriteDate(new Timestamp(new Date().getTime()));
            earmarkRepo.saveAndFlush(earmarkEntity);
        }
        return new Result(ResultCode.SUCCESS);
    }

    /**
     * @Method : setWriteState
     * @Description : 设置写入时间
     * @Params : [ids, state]
     * @Return : com.huimv.manage.util.Result
     * @Author : ZhuoNing
     * @Date : 2021/10/30
     * @Time : 0:02
     */
    @Override
    public Result setWriteState(String ids, int state) {
        String[] idArray = ids.split(",");
        for (int a = 0; a < idArray.length; a++) {
            Optional optional = earmarkRepo.findById(Integer.parseInt(idArray[a]));
            EtEarmarkEntity earmarkEntity = (EtEarmarkEntity) optional.get();
            earmarkEntity.setSetWriteState(state);
            earmarkEntity.setSetWriteDate(new Timestamp(new Date().getTime()));
            earmarkRepo.saveAndFlush(earmarkEntity);
        }
        return new Result(ResultCode.SUCCESS);
    }

    /**
     * @Method : setPrintState
     * @Description : 设置打码状态
     * @Params : [ids, state]
     * @Return : com.huimv.manage.util.Result
     * @Author : ZhuoNing
     * @Date : 2021/10/30
     * @Time : 0:04
     */
    @Override
    public Result setPrintState(String ids, int state) {
        String[] idArray = ids.split(",");
        for (int a = 0; a < idArray.length; a++) {
            Optional optional = earmarkRepo.findById(Integer.parseInt(idArray[a]));
            EtEarmarkEntity earmarkEntity = (EtEarmarkEntity) optional.get();
            earmarkEntity.setSetPrintState(state);
//            earmarkEntity.setSetPrintDate(new Timestamp(new Date().getTime()));
            earmarkEntity.setSetPrintDate(new Timestamp(System.currentTimeMillis()));
            earmarkRepo.saveAndFlush(earmarkEntity);
        }
        return new Result(ResultCode.SUCCESS);
    }

    //过滤已经存在的耳标号
    private JSONArray filterEarmark(JSONArray downloadEarmarkJa) {
        JSONArray newEarmarkJa = new JSONArray();
        for (int a = 0; a < downloadEarmarkJa.size(); a++) {
            JSONObject downloadEarmarkJo = downloadEarmarkJa.getJSONObject(a);
            String earmarkId = downloadEarmarkJo.getString("earmarkId");

            EtEarmarkEntity earmarkEntity = new EtEarmarkEntity();
            earmarkEntity.setEarmarkId(earmarkId);
            ExampleMatcher matcher = ExampleMatcher.matching()
                    .withMatcher("earmark_id", ExampleMatcher.GenericPropertyMatchers.contains());
            Example example = Example.of(earmarkEntity, matcher);
            if (!earmarkRepo.exists(example)) {
                newEarmarkJa.add(downloadEarmarkJo);
            }
        }
        return newEarmarkJa;
    }

    /**
     * @Method : exportEarmarkByApply
     * @Description : 通过任务号导出耳标号
     * @Params : [applyId]
     * @Return : void
     * @Author : ZhuoNing
     * @Date : 2021/11/9
     * @Time : 14:11
     */
    @Override
    public void exportEarmarkByApply(Integer applyId, Integer printState, Integer quantity, HttpServletResponse response, HttpServletRequest request) throws IOException, ArchiveException {
        Specification<EtEarmarkEntity> specific = (Specification<EtEarmarkEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != applyId) {
                predList.add(criteriaBuilder.equal(root.get("applyId").as(Integer.class), applyId));
            }
            if (null != printState && printState != 0) {
                predList.add(criteriaBuilder.equal(root.get("printState").as(Integer.class), printState));
            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.asc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        //
        List<EtEarmarkEntity> dataList = earmarkRepo.findAll(specific);
        log.info("导出数据数量=" + dataList.size());
        //
//        excelUtil.exportEarmarkExcel(response, Const.EXPORT_EARMARK_EXCEL, String.valueOf(applyId), dataList, quantity);
        // test
//        excelUtil.exportEarmarkExcel(String.valueOf(applyId),dataList,quantity);
        if(dataList.size()>0){
            //导出耳标数据到文本文件
            txtUtil.exportEarmarkToTxt(response,String.valueOf(applyId), dataList, quantity,request);
        }else{
            //导出空数据到文本文件
            txtUtil.exportEmptyEarmarkToTxt(response,String.valueOf(applyId),request);
        }
    }

    //--old
    public void exportEarmarkByApply_old(Integer applyId, Integer printState, Integer quantity, HttpServletResponse response) throws IOException {
        Specification<EtEarmarkEntity> specific = (Specification<EtEarmarkEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != applyId) {
                predList.add(criteriaBuilder.equal(root.get("applyId").as(Integer.class), applyId));
            }
            if (null != printState && printState != 0) {
                predList.add(criteriaBuilder.equal(root.get("printState").as(Integer.class), printState));
            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.asc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        //
        List<EtEarmarkEntity> dataList = earmarkRepo.findAll(specific);
        log.info("导出数据数量=" + dataList.size());
        //
        excelUtil.exportEarmarkExcel(response, Const.EXPORT_EARMARK_EXCEL, String.valueOf(applyId), dataList, quantity);
        // test
//        excelUtil.exportEarmarkExcel(String.valueOf(applyId),dataList,quantity);
    }

    @Override
    public void exportEarmarkByEarmark(String ids, Integer printState, Integer quantity, HttpServletResponse response) throws IOException {
        Specification<EtEarmarkEntity> specific = (Specification<EtEarmarkEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != ids && !ids.trim().equalsIgnoreCase("null")) {
                String[] idsArray = ids.split(",");
                Integer[] idArrInt = new Integer[idsArray.length];
                for (int a = 0; a < idsArray.length; a++) {
                    idArrInt[a] = Integer.parseInt(idsArray[a]);
                }
//                predList.add(criteriaBuilder.equal(root.get("id").as(Integer.class), applyId));
                predList.add(root.get("id").as(Integer.class).in(idArrInt));
            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.asc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        //
        List<EtEarmarkEntity> dataList = earmarkRepo.findAll(specific);
        log.info("导出数据数量:" + dataList.size());
        String applyId = null;
        if (dataList.size() > 0) {
            EtEarmarkEntity earmarkEntity = dataList.get(0);
            applyId = earmarkEntity.getApplyId();
        }
        //
        excelUtil.exportEarmarkExcel(response, Const.EXPORT_EARMARK_EXCEL, applyId, dataList, quantity);
        // test
//        excelUtil.exportEarmarkExcel(applyId,dataList,quantity);
    }

    //导出耳标数据到文本文件中
    @Override
    public void exportEarmarkByApplyToTxt(Integer applyId, Integer printState, Integer quantity, HttpServletResponse response) {


    }

    @Override
    public void exportTest1(Integer applyId, Integer printState, Integer quantity, HttpServletResponse response, HttpServletRequest request) throws IOException, ArchiveException {
//        String fileName = "fileName" + ".txt";
//        String content = "写入txt的内容";
//        response.setContentType("text/plain");
//
//        try {
//            response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8"));
//        } catch (UnsupportedEncodingException e) {
//            e.printStackTrace();
//        }
//
//        ServletOutputStream outputStream = null;
//        BufferedOutputStream buffer = null;
//
//        try {
//            outputStream = response.getOutputStream();
//            buffer = new BufferedOutputStream(outputStream);
//            buffer.write(content.getBytes("UTF-8"));
//            buffer.flush();
//            buffer.close();
//            outputStream.close();
//        } catch (IOException e) {
//            e.printStackTrace();
//        }
        //定义根路径
        //Linux
        String rootPath = request.getRealPath("/");
        //Windows
//        String rootPath = "D:\\";
        String basePath = rootPath + "temp_download";
        System.out.println("basePath>>" + basePath);
        //创建文件
        File file = new File(basePath);
        //判断文件是否存在，如果不存在，则创建此文件夹
        System.out.println("file.exists() 11.>>" + file.exists());
        if (!file.exists()) {
            file.mkdir();
        }

//        String basePath = getResourceBasePath(request);
//        System.out.println("basePath>>"+basePath);
//        String studentResourcePath = new File(basePath, "student/student.txt").getAbsolutePath();
//        File txtFile1 = new File(basePath, "eartag\\earmark_1.txt");
        File txtFile1 = new File(basePath, "eartag/earmark_1.txt");
        System.out.println("txtFile1.exists()>>" + txtFile1.exists());
//        String studentResourcePath = txtFile1.getAbsolutePath();
//        System.out.println("studentResourcePath = " + studentResourcePath);
        // 保证目录一定存在
//        ensureDirectory(studentResourcePath);

        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(txtFile1)));
//        for (Student student : students) {
//            StringBuffer buffer = new StringBuffer();
//            buffer.append(student.getName());
//            buffer.append("\t");
//            buffer.append(student.getAge());
//            buffer.append("\t");
//            buffer.append(student.getSex());
//            buffer.append("\r\n");
//            writer.write(buffer.toString());
//        }
        String text = "测试内容";
        writer.write(text);
        writer.flush();
        writer.close();
//        File txtFile2 = new File(basePath, "eartag/earmark_2.txt");
//        System.out.println("txtFile2.exists()>>"+txtFile2.exists());
//        File txtFile3 = new File(basePath, "eartag/earmark_3.txt");
//        System.out.println("txtFile3.exists()>>"+txtFile3.exists());
        List<File> fileList = new ArrayList();
        fileList.add(txtFile1);
//        fileList.add(txtFile2);
//        fileList.add(txtFile3);

//        String fileName = "耳标数据下载" + "_" + dateUtil.formatLongTimeForTime(new Date().getTime());
        String fileName = "耳标数据下载";
        String zipFileName = fileName + ".zip";
        System.out.println("zipFileName>>" + zipFileName);

//        File filex = new File(basePath + "\\newFile.txt");
//        System.out.println("###### filex.exists>>"+filex.exists());
//        String resourcePath = txtFile1.getAbsolutePath();
//        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(resourcePath)));
//        String text = "测试内容";
//        writer.write(text);
//        writer.flush();
//        writer.close();

        // 打成压缩包
//        File zipFile = null;
//        String zipPath = basePath + "\\eartag\\" + zipFileName;
        String zipPath = basePath + "/eartag/" + zipFileName;
        System.out.println("zipPath>>" + zipPath);
        File zipFile = new File(zipPath);
        System.out.println("zipFileexists.exists>>" + zipFile.exists());
        FileOutputStream zipFos = new FileOutputStream(zipFile);
        ArchiveOutputStream archOut = new ArchiveStreamFactory().createArchiveOutputStream(ArchiveStreamFactory.ZIP, zipFos);
        if (archOut instanceof ZipArchiveOutputStream) {
            ZipArchiveOutputStream zos = (ZipArchiveOutputStream) archOut;
            for (File fileFile : fileList) {
                ZipArchiveEntry zipEntry = new ZipArchiveEntry(fileFile, fileFile.getName());
                zos.putArchiveEntry(zipEntry);
                zos.write(FileUtils.readFileToByteArray(fileFile));
                zos.closeArchiveEntry();
                zos.flush();
            }
            zos.close();
        }
        // 输出到客户端
        OutputStream out = null;
        out = response.getOutputStream();
        response.reset();
        response.setHeader("Content-Disposition", "attachment;filename=" + new String(zipFileName.getBytes("GB2312"), "ISO-8859-1"));
        response.setContentType("application/octet-stream; charset=utf-8");
        response.setCharacterEncoding("UTF-8");
        out.write(FileUtils.readFileToByteArray(zipFile));
        out.flush();
        out.close();
        // 输出客户端结束后，删除压缩包
        if (zipFile.exists()) {
            zipFile.delete();
        }
    }

    @Override
    public void exportTest2(Integer applyId, Integer printState, Integer quantity, HttpServletResponse response, HttpServletRequest request) throws IOException {
        String basePath = getResourceBasePath(request);
        System.out.println("basePath>>" + basePath);
//        String studentResourcePath = new File(basePath, "student/student.txt").getAbsolutePath();
        File txtFile = new File(basePath, "eartag/earmark_1.txt");
        String studentResourcePath = txtFile.getAbsolutePath();
        System.out.println("studentResourcePath = " + studentResourcePath);
        // 保证目录一定存在
        ensureDirectory(studentResourcePath);

        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(studentResourcePath)));
//        for (Student student : students) {
//            StringBuffer buffer = new StringBuffer();
//            buffer.append(student.getName());
//            buffer.append("\t");
//            buffer.append(student.getAge());
//            buffer.append("\t");
//            buffer.append(student.getSex());
//            buffer.append("\r\n");
//            writer.write(buffer.toString());
//        }
        String text = "测试内容";
        writer.write(text);
        writer.flush();
        writer.close();

        response.setHeader("content-type", "application/octet-stream");
        response.setContentType("application/octet-stream");
        response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=download.zip");
        //
        ZipUtils.downloadZip(response.getOutputStream(), txtFile);
    }

    @Override
    public void export1(Integer applyId, Integer printState, Integer quantity, HttpServletResponse response, HttpServletRequest request) throws IOException, ArchiveException {
        //定义根路径
        //Linux
        String rootPath = request.getRealPath("/");
        //Windows
//        String rootPath = "D:\\";
        String basePath = rootPath + "temp_download";
        System.out.println("basePath>>" + basePath);
        //创建文件
        File file = new File(basePath);
        //判断文件是否存在，如果不存在，则创建此文件夹
        System.out.println("file.exists() 11.>>" + file.exists());
        if (!file.exists()) {
            file.mkdir();
        }
        File t1File = new File(basePath + "/t1.txt");
        if (!t1File.exists()) {
            // t1File.delete();
            t1File.createNewFile();
        }
        System.out.println("t1File.exists()>>"+t1File.exists());
        BufferedWriter bw = new BufferedWriter(new FileWriter(t1File));
        String ss = "测试数据";
        bw.write(ss);
        bw.newLine();  //换行用
        //关闭流
        bw.close();
        System.out.println("写入成功");

        List<File> txtList = new ArrayList();
        txtList.add(t1File);
        for(int a=0;a<txtList.size();a++){
            System.out.println(">>"+txtList.get(a));
        }
        ///////////////////////////////////////////////////////////////////////////////////////////////////////
        String name = "耳标文件压缩包下载";
        String fileName = name + "_" + dateUtil.formatLongTimeForTime(new Date().getTime());
        String zipFileName = fileName + ".zip";
        File zipFile = null;
//        String path = rootPath + "temp_download";
        // 打成压缩包
        zipFile = new File(basePath + "/" + zipFileName);
        FileOutputStream zipFos = new FileOutputStream(zipFile);
        ArchiveOutputStream archOut = new ArchiveStreamFactory().createArchiveOutputStream(ArchiveStreamFactory.ZIP, zipFos);
        if (archOut instanceof ZipArchiveOutputStream) {
            ZipArchiveOutputStream zos = (ZipArchiveOutputStream) archOut;
            for (File txtFile : txtList) {
                ZipArchiveEntry zipEntry = new ZipArchiveEntry(txtFile, txtFile.getName());
                zos.putArchiveEntry(zipEntry);
                zos.write(FileUtils.readFileToByteArray(txtFile));
                zos.closeArchiveEntry();
                zos.flush();
            }
            zos.close();
        }
        // 输出到客户端
        OutputStream out = null;
        out = response.getOutputStream();
        response.reset();
        response.setHeader("Content-Disposition", "attachment;filename=" + new String(zipFileName.getBytes("GB2312"), "ISO-8859-1"));
        response.setContentType("application/octet-stream; charset=utf-8");
        response.setCharacterEncoding("UTF-8");
        out.write(FileUtils.readFileToByteArray(zipFile));
        out.flush();
        out.close();

        System.out.println("zipFile.exists()>>"+zipFile.exists());
        // 输出客户端结束后，删除压缩包
//        if (zipFile.exists()) {
//            zipFile.delete();
//        }

    }

    @Override
    public void exportEarmarkByApply(Integer applyId, Integer printState, Integer writeState, Integer quantity, HttpServletResponse response, HttpServletRequest request) throws IOException, ArchiveException {
        Specification<EtEarmarkEntity> specific = (Specification<EtEarmarkEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            //applyId
            if (null != applyId) {
                predList.add(criteriaBuilder.equal(root.get("applyId").as(Integer.class), applyId));
            }
            //printState
            if (null != printState && printState != 0) {
                predList.add(criteriaBuilder.equal(root.get("printState").as(Integer.class), printState));
            }
            //writeState
            if (null != writeState) {
                predList.add(criteriaBuilder.equal(root.get("setWriteState").as(Integer.class), writeState));
            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.asc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        //
        List<EtEarmarkEntity> dataList = earmarkRepo.findAll(specific);
        log.info("导出数据数量=" + dataList.size());
        if(dataList.size()>0){
            //导出耳标数据到文本文件
            txtUtil.exportEarmarkToTxt(response,String.valueOf(applyId), dataList, quantity,request);
        }else{
            //导出空数据到文本文件
            txtUtil.exportEmptyEarmarkToTxt(response,String.valueOf(applyId),request);
        }
    }

    @Override
    public String baindEartag(String aplyID) {
        List<EtEarmarkEntity> etEarmarkEntities = earmarkRepo.listEarmarkByApplyASC(aplyID);
        int eartagSize = etEarmarkEntities.size();
        List<EtPackageEntity> etPackageEntities =  packageRepo.listByApply(aplyID);
        int packageSize = etPackageEntities.size();
//        int num =  eartagSize /packageSize;
        //16 48 96
        int num = 16;
        Integer count =0;

        for (EtPackageEntity etPackageEntity : etPackageEntities) {
            for (int i = num*count; i<num *(count+1);i++) {
                Integer packageId = etPackageEntity.getPackageId();
                if (i < eartagSize){
                    EtEarmarkEntity etEarmarkEntity = etEarmarkEntities.get(i);
                    etEarmarkEntity.setPackageId(packageId+"");
                    earmarkRepo.saveAndFlush(etEarmarkEntity);
                }

            }
            count++;
        }
        return null;

    }

    /**
     * 获取项目根路径
     *
     * @return
     */
    private static String getResourceBasePath(HttpServletRequest request) {
//        // 获取跟目录
//        File path = null;
//        try {
//            path = new File(ResourceUtils.getURL("classpath:").getPath());
//        } catch (FileNotFoundException e) {
//            // nothing to do
//        }
//        if (path == null || !path.exists()) {
//            path = new File("");
//        }
//
//        String pathStr = path.getAbsolutePath();
//        // 如果是在eclipse中运行，则和target同级目录,如果是jar部署到服务器，则默认和jar包同级
//        pathStr = pathStr.replace("\\target\\classes", "");

        //定义根路径
        String rootPath = request.getRealPath("/");
        //创建文件
        File file = new File(rootPath + "temp_download");
        //判断文件是否存在，如果不存在，则创建此文件夹
        if (!file.exists()) {
            file.mkdir();
        }
        String pathStr = file.getAbsolutePath();
        System.out.println("pathStr>>" + pathStr);
        return pathStr;
    }

    /**
     * 保证拷贝的文件的目录一定要存在
     *
     * @param filePath 文件路径
     */
    public static void ensureDirectory(String filePath) {
        if (StringUtils.isEmpty(filePath)) {
            return;
        }
        filePath = replaceSeparator(filePath);
        if (filePath.indexOf("/") != -1) {
            filePath = filePath.substring(0, filePath.lastIndexOf("/"));
            File file = new File(filePath);
            if (!file.exists()) {
                file.mkdirs();
            }
        }
    }

    /**
     * 将符号“\\”和“\”替换成“/”,有时候便于统一的处理路径的分隔符,避免同一个路径出现两个或三种不同的分隔符
     *
     * @param str
     * @return
     */
    public static String replaceSeparator(String str) {
        return str.replace("\\", "/").replace("\\\\", "/");
    }

}
